#!ocaml unix.cma

let framac = "/usr/local/bin/frama-c"
let options = [|
  framac;
  "trap.c";
  "-cpp-extra-args= \
   -I ../.. \
   -I /usr/include/glib-2.0/ \
   -I /usr/lib/x86_64-linux-gnu/glib-2.0/include/ \
   -DTAG_ENCRYPTION=G_CHECKSUM_SHA1 -D_GNU_SOURCE";
  "-main=TrapHandler";
  "-pp-annot";
  "-wp-par=8";
  "-machdep=x86_64";
  "-wp-out=session";
  "-warn-signed-downcast";
  "-warn-signed-overflow";
  "-warn-unsigned-downcast";
  "-warn-unsigned-overflow";
  "-no-frama-c-stdlib";
|]


let script = [|
  "-wp-fct=CheckRAMAccess,ZVMReadHandle,ZVMWriteHandle,\
   NaClUserToSys,NaClUserToSysAddrNullOkay,TrapHandler";
  "-wp-prover=CVC4:1.3"; "-wp-timeout=10"; "-wp";
  "-then";
  "-wp-fct=CheckRAMAccess";
  "-wp-prover=alt-ergo"; "-wp-timeout=10"; "-wp";
  "-then";
  "-wp-fct=CheckRAMAccess";
  "-wp-prover=Alt-Ergo:0.95.2"; "-wp-timeout=10"; "-wp";
  "-then";
  "-wp-fct=CheckRAMAccess,ZVMReadHandle,ZVMWriteHandle,\
   NaClUserToSys,NaClUserToSysAddrNullOkay";
  "-wp-prover=CVC4:1.3"; "-wp-timeout=10"; "-wp-rte"; "-wp";
  "-then";
  "-no-rte"; "-no-wp";
  "-report";
|]

let () =
  let options =
    if Array.length Sys.argv <= 2 then options
    else
      Array.append options
        (Array.sub Sys.argv 2 (Array.length Sys.argv-2))
  in
  match try Sys.argv.(1) with _ -> "cmd" with
  | "cmd" -> Unix.execv framac options
  | "gui" -> Unix.execv (framac^"-gui") options
  | "script" -> Unix.execv framac (Array.append options script)
  | "gui-script" -> Unix.execv (framac^"-gui") (Array.append options script)
  | _ -> Printf.eprintf "Error: unknown command"
